"""
-*- coding: utf-8 -*-
@Time: 2023/8/15 20:37
@Author: HW
@FileName: gepd.py
@Mail: leowu@zju.edu.cn
@Software: picpalette
"""

from pygds2 import *


class gepd_lpin(cell):
    def __init__(self, label='V1', band='O', mode='TEM', w_IO=0.41, w_Ge=0.6, l_Ge=20.0, l_IO=15.0):
        name = 'GePD_LPIN_{}B{}_{}_W{}_L{}_{}'.format(band, mode, Technology, w_Ge, l_Ge, label)
        cell.__init__(self, name=name, port='N')

        self.addport(pos=0, w=w_IO, wgtype=TECH_Lyr.WG3, type='IO')

        e_Si_Ge = 0.2
        w_Si = w_Ge + e_Si_Ge * 2
        l_Si = l_Ge + e_Si_Ge * 2
        l_Con = 2.0
        g_pn = w_Ge - 0.3
        g_contact = w_Si + 2
        # Si
        self.swg_cld(lyr=TECH_Lyr.WG3, w=w_IO, l=l_Con)
        self.twg_cld(lyr=TECH_Lyr.WG3, w_in=w_IO, w_out=w_Si, l=l_IO)
        self.swg_cld(lyr=TECH_Lyr.WG1, w=w_Si, l=l_Si, w_tre=3)
        self.twg_cld(lyr=TECH_Lyr.WG3, w_in=w_Si, w_out=w_IO, l=l_IO)
        self.swg_cld(lyr=TECH_Lyr.WG3, w=w_IO, l=l_Con)
        self.addport(pos=l_Con + l_IO + e_Si_Ge, type='N')
        # Ge
        self.swg(lyr=TECH_Lyr.GE, w=w_Ge, l=l_Ge)
        # Si N Implant
        l_pd_in = l_Con + l_IO
        self.shiftport(pos=l_pd_in + g_pn * 0.5j, type=1)
        self.rect(lyr=TECH_Lyr.NP, w=l_Si, h=3.5, pos='left_down')
        self.shiftport(pos=l_pd_in + g_contact * 0.5j, type=1)
        self.rect(lyr=TECH_Lyr.NH, w=l_Si, h=2.0, pos='left_down')
        self.rect(lyr=TECH_Lyr.M1, w=l_Si, h=4.0, pos='left_down')
        self.shiftport(pos=0.3 + 0.3j, type=0)
        self.rect(lyr=TECH_Lyr.CS, w=l_Si - 0.6, h=1, pos='left_down')
        # Si P Implant
        self.shiftport(pos=l_pd_in - g_pn * 0.5j, type=1)
        self.rect(lyr=TECH_Lyr.PP, w=l_Si, h=3.5, pos='left_up')
        self.shiftport(pos=l_pd_in - g_contact * 0.5j, type=1)
        self.rect(lyr=TECH_Lyr.PH, w=l_Si, h=2.0, pos='left_up')
        self.rect(lyr=TECH_Lyr.M1, w=l_Si, h=4.0, pos='left_up')
        self.shiftport(pos=0.3 - 0.3j, type=0)
        self.rect(lyr=TECH_Lyr.CS, w=l_Si - 0.6, h=1, pos='left_up')


class gepd_vpin(cell):
    def __init__(self, label='V1', band='O', mode='TEM', type='GN', w_IO=0.41, w_Si_Ge=1.6, w_Ge=4, l_Ge=20.0,
                 l_IO=15.0):
        name = 'GePD_VPIN_{}B{}_{}_{}_W{}_L{}_{}'.format(band, mode, type, Technology, w_Ge, l_Ge, label)
        cell.__init__(self, name=name, port='N')

        self.addport(pos=0, w=w_IO, wgtype=TECH_Lyr.WG3, type='IO')

        e_Si_Ge = 0.2
        w_Si = w_Ge + e_Si_Ge * 2
        l_Si = l_Ge + e_Si_Ge * 2
        l_Con = 2.0
        g_pn = w_Ge - 0.3
        g_contact = w_Si + 2
        if type == 'GN':
            lyr_IMP_Ge = TECH_Lyr.GN
            lyr_IMP_Si1 = TECH_Lyr.PP
            lyr_IMP_Si2 = TECH_Lyr.PH
        else:
            lyr_IMP_Ge = TECH_Lyr.GP
            lyr_IMP_Si1 = TECH_Lyr.NP
            lyr_IMP_Si2 = TECH_Lyr.NH

        # Si
        self.swg_cld(lyr=TECH_Lyr.WG3, w=w_IO, l=l_Con)
        self.twg_cld(lyr=TECH_Lyr.WG3, w_in=w_IO, w_out=w_Si_Ge, l=l_IO)
        self.swg_cld(lyr=TECH_Lyr.WG1, w=w_Si, l=l_Si, w_tre=3)
        self.twg_cld(lyr=TECH_Lyr.WG3, w_in=w_Si_Ge, w_out=w_IO, l=l_IO)
        self.swg_cld(lyr=TECH_Lyr.WG3, w=w_IO, l=l_Con)
        self.addport(pos=l_Con + l_IO + e_Si_Ge, type='N')
        # Ge
        self.swg(lyr=TECH_Lyr.GE, w=w_Ge, l=l_Ge)
        # Si Implant and contact
        l_pd_in = l_Con + l_IO
        self.shiftport(pos=l_pd_in, type=1)
        self.swg(lyr=lyr_IMP_Si1, w=g_contact + 4, l=l_Si)
        self.shiftport(pos=l_pd_in + g_contact * 0.5j, type=1)
        self.rect(lyr=lyr_IMP_Si2, w=l_Si, h=2.0, pos='left_down')
        self.rect(lyr=TECH_Lyr.M1, w=l_Si, h=2.0, pos='left_down')
        self.shiftport(pos=0.3 + 0.3j, type=0)
        self.rect(lyr=TECH_Lyr.CS, w=l_Si - 0.6, h=1, pos='left_down')
        self.shiftport(pos=l_pd_in - g_contact * 0.5j, type=1)
        self.rect(lyr=lyr_IMP_Si2, w=l_Si, h=2.0, pos='left_up')
        self.rect(lyr=TECH_Lyr.M1, w=l_Si, h=2.0, pos='left_up')
        self.shiftport(pos=0.3 - 0.3j, type=0)
        self.rect(lyr=TECH_Lyr.CS, w=l_Si - 0.6, h=1, pos='left_up')

        # Ge Implant and contact
        self.shiftport(pos=l_pd_in + l_Si / 2, type=1)
        self.rect(lyr=lyr_IMP_Ge, w=l_Ge - 0.6, h=w_Ge - 0.6, pos='mid')
        self.shiftport(pos=l_pd_in + l_Si / 2, type=1)
        self.rect(lyr=TECH_Lyr.CG, w=l_Ge - 0.6 - 0.6, h=1.0, pos='mid')
        self.rect(lyr=TECH_Lyr.M1, w=l_Si, h=w_Ge, pos='mid')


if __name__ == '__main__':
    tmp = gepd_lpin()
    layout('gdsii\\' + tmp.name)
    tmp = gepd_vpin()
    layout('gdsii\\' + tmp.name)
